import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;

public class BrushDrawTool extends DrawTool {
	int [] [] sprite;
	boolean invert = false;
	BrushDrawTool(PaintPanel paintPanel) {
		super(paintPanel);
		init();
	}
	BrushDrawTool(PaintPanel paintPanel,boolean inversion) {
		super(paintPanel);
		invert = inversion;
		init();
	}
	void init() {
		int size = 10;
		int max = 30;
		if (invert) {
			max *= -1;
		}
		 
		int val = max / size;
		sprite = new int [size][size];
		for (int x = 0; x <= size/2; x++) {
			val = (x * max) / (size/2);
			for (int y = x; y < (size - 2*x); y++) {
				
				sprite[x][y] 		= val;
				sprite[y][x] 		= val;
				sprite[size - x - 1][y] = val;
				sprite[y][size - x - 1] = val;
			}
		}

	}
	void actionPoint(int x, int y) {
		addDot(x,y);	
	}
	void actionLine(int x1, int y1, int x2, int y2) {
		addLine(x1,y1,x2,y2);
	}
	void addDot(int x,int y) {
		paintPanel.addRGB(x,y,30,30,30);
		paintPanel.addRGB(x+1,y+1,10,10,10);
		paintPanel.addRGB(x-1,y-1,10,10,10);
		paintPanel.addRGB(x-1,y+1,10,10,10);
		paintPanel.addRGB(x+1,y-1,10,10,10);
		paintPanel.addRGB(x,y-1,20,20,20);
		paintPanel.addRGB(x,y+1,20,20,20);
		paintPanel.addRGB(x+1,y,20,20,20);
		paintPanel.addRGB(x-1,y,20,20,20);
	}
	void addSprite(int x,int y) {
		int xmin = x - sprite.length/2;
		int ymin = y - sprite.length/2;
		for (int Y = 0; Y < sprite.length; Y++) {
			for (int X = 0; X < sprite.length; X++) {
				int k = sprite[Y][X];
				paintPanel.addRGB(xmin+X,ymin+Y,k,k,k);
			}
		}
	}
	void addLine(int x1, int y1, int x2, int y2) {
		int xdiff = Math.max(x2,x1) - Math.min(x2,x1);	
		int ydiff = Math.max(y2,y1) - Math.min(y1,y2);
		int tmp = 0;
		if (((xdiff > ydiff) && (x1 > x2)) || ((xdiff < ydiff) && (y1 > y2))) {
			tmp = y2;
			y2 = y1;
			y1 = tmp;
			tmp = x2;
			x2 = x1;
			x1 = tmp;
		}
		
		double yinc = 1.0*(y2 - y1) / (x2 - x1);
		double xinc = 1.0*(x2 - x1) / (y2 - y1);
		double X = 0.0;
		double Y = 0.0;
		if (xdiff > ydiff) {
			Y = (double)y1;
			for (int x = x1; x <= x2; x++) {
				Y += yinc;
				int y = (int)Math.round(Y);
				addSprite(x,y);
			}
		} else {
			X = (double)x1;
			for (int y = y1; y <= y2; y++) {
				X += xinc;
				int x = (int)Math.round(X);
				addSprite(x,y);
			}
		}
	}
	String getName() {
		if (invert) {
			return "Erase";
		} else {
			return "Paint";
		}
	}
}
