public class fft {
	public static boolean fft ( int NumSamples, boolean InverseTransform, float [] RealIn, float [] ImagIn, float [] RealOut, float [] ImagOut ) {
		int NumBits;    /* Number of bits needed to store indices */
    		int i=0, j=0, k=0, n=0;
		int BlockSize=0, BlockEnd=0;
    		float angle_numerator = (float) ( 2.0 * Math.PI);
		float tr, ti;     /* temp real, temp imaginary */
		if (!IsPowerOfTwo(NumSamples)) {
			return false;
		}
		if ( InverseTransform )	{
			angle_numerator = -angle_numerator;
		}
		
		NumBits = NumberOfBitsNeeded ( NumSamples );
		for ( i = 0; i < NumSamples; i++) {
			j = ReverseBits ( i, NumBits );
		        RealOut[j] = RealIn[i];
	        	ImagOut[j] = (ImagIn == null) ? 0.0f : ImagIn[i];
		}
		BlockEnd = 1;
		for ( BlockSize = 2; BlockSize <= NumSamples; BlockSize <<= 1 ) {
			float delta_angle = angle_numerator / (float)BlockSize;
			float sm2 = (float)Math.sin ( -2 * delta_angle );
			float sm1 = (float)Math.sin ( -delta_angle );
			float cm2 = (float)Math.cos ( -2 * delta_angle );
			float cm1 = (float)Math.cos ( -delta_angle );
			float w = 2 * cm1;
			float [] ar = new float[3];
			float [] ai = new float[3];
			float temp;
		        for ( i=0; i < NumSamples; i += BlockSize ) {
			    ar[2] = cm2;
			    ar[1] = cm1;
			
			    ai[2] = sm2;
			    ai[1] = sm1;
			
			    for ( j=i, n=0; n < BlockEnd; j++, n++ )
			    {
			        ar[0] = w*ar[1] - ar[2];
			        ar[2] = ar[1];
			        ar[1] = ar[0];
			
			        ai[0] = w*ai[1] - ai[2];
			        ai[2] = ai[1];
			        ai[1] = ai[0];
			
			        k = j + BlockEnd;
			        tr = ar[0]*RealOut[k] - ai[0]*ImagOut[k];
			        ti = ar[0]*ImagOut[k] + ai[0]*RealOut[k];
			
			        RealOut[k] = RealOut[j] - tr;
			        ImagOut[k] = ImagOut[j] - ti;
			
			        RealOut[j] += tr;
			        ImagOut[j] += ti;
			    }
			}

		        BlockEnd = BlockSize;
    		}

		if ( InverseTransform ) {
			float denom = (float)NumSamples;

			for ( i=0; i < NumSamples; i++ )
			{
			    RealOut[i] /= denom;
			    ImagOut[i] /= denom;
			}
    		}

		return true;
	}
	static float [] [] cos = null;
	public static void initCos(int NumSamples) {
		float PI2 = (float)( 2.0*Math.PI);
		float C = PI2 / NumSamples;
		float A = (float)Math.log((NumSamples * NumSamples) / C) / ((float)NumSamples);
		if (cos == null || cos.length != NumSamples) {
			cos = new float [NumSamples] [NumSamples];
			for (int k = 0; k < NumSamples; k++) {
				for (int t = 0; t < NumSamples; t++) {
					cos[k][t] = (float)Math.cos(C*t*Math.exp(A*k/10));
				}
			}
		}
	}
	public static boolean cosinualfft(float [] RealIn, float [] RealOut) {
		int NumSamples = RealOut.length;
		initCos(NumSamples);
		if (!IsPowerOfTwo(NumSamples)) {
			return false;
		}
		for (int i = 0; i < NumSamples; i++) {
			RealOut[i] = 0.0f;
		}
		for (int k = 0; k < NumSamples; k++) {
			float amplitude = RealIn[k];
			if (Math.abs(amplitude) > .1) {
				for (int t = 0; t < NumSamples; t++) {
					RealOut[t] += amplitude*cos[k][t];
				}
			} 

		}
		return true;
	}
	public static boolean fftDiscrete ( int NumSamples, boolean InverseTransform, float [] RealIn, float [] RealOut ) {
        	float f0 = 1.0f / NumSamples;
		float PI2 = 2.0f*(float)Math.PI;
		float C = f0*PI2;
		if (!IsPowerOfTwo(NumSamples)) {
			return false;
		}
		if (!InverseTransform) {
			for (int t = 0; t < NumSamples; t++) {
				for (int k  = 0; k < NumSamples; k++) {
			                RealOut[t] += RealIn[k]*Math.cos(C*t*k);
				}
        		}
		} else {
			for (int t = 0; t < NumSamples; t++) {
				for (int k  = 0; k < NumSamples; k++) {
			                RealOut[t] -= RealIn[k]/Math.cos(C*t*k); 
				}
        		}
		}
		return true;
	}
	public static int ReverseBits ( int index, int NumBits )
	{
	    int i, rev;
	
	    for ( i=rev=0; i < NumBits; i++ )
	    {
	        rev = (rev << 1) | (index & 1);
	        index >>= 1;
	    }
	
	    return rev;
	}

	public static boolean IsPowerOfTwo ( int x )
	{
	    if ( x < 2 )
	        return false;
	
	    if ( (x & (x-1)) > 0 )        // Thanks to 'byang' for this cute trick!
	        return false;
	
	    return true;
	}

	public static int NumberOfBitsNeeded ( int PowerOfTwo )
	{
	    int i;
	
	    if ( PowerOfTwo < 2 )
	    {
		return 1;
	    }
	
	    for ( i=0; ; i++ )
	    {
	        if ( (PowerOfTwo & (1 << i)) > 0 )
	            return i;
	    }
	}
	public static float magnitude(float real,float imag) {
		return (float)Math.sqrt(real*real+imag*imag);
	}
	public static float angle(float real,float imag) {
		return (float) Math.atan2(real,imag);
	}
	public static int anglei(float real,float imag) {
		return new Double(Math.atan2(real,imag)).intValue();
	}
	public static int magnitudei(float real,float imag) {
		return new Double(Math.sqrt(real*real+imag*imag)).intValue();
	}
	public static float toReal(float magnitude,float angle) {
		return magnitude*(float)Math.cos(angle);
	}
	public static float toImag(float magnitude,float angle) {
		return magnitude*(float)Math.sin(angle);
	}
}
